using System;
using CMS.ContactManagement;
using Twilio;
using CMS.DataEngine;
using CMS.SiteProvider;
using Twilio.Rest.Api.V2010.Account;
using Twilio.Types;
using CMS.EventLog;
using CMS;

// uncomment if using in Kentico web site
// [assembly: RegisterCustomClass("TwilioSMS.TwilioSMSAutomationAction", typeof(TwilioSMS.TwilioSMSAutomationAction))]
namespace TwilioSMS
{
    public class TwilioSMSAutomationAction : ContactAutomationAction
    {
        public override void Execute()
        {
            // Get Parameters from the Workflow Automation Step config in Kentico
            String toNumber = GetResolvedParameter<string>("ToNumber", String.Empty);
            String messageText = GetResolvedParameter<string>("MessageText", String.Empty);

            // Remove extraneous characters in phone number
            toNumber = toNumber.Replace(" ", "").Replace("-", "").Replace(")", "").Replace("(", "").Replace("+", "").Replace(".", "").Replace("x", "");

            // Check for empty to number
            if (String.IsNullOrEmpty(toNumber))
            {                
                EventLogProvider.LogEvent(EventType.ERROR, "TwilioSMS", "SEND", "To Number is empty.  Either Contact's mobile number is empty or there is no number assigned to the Twilio Send SMS Automation Action in the workflow.");
                return;
            }
            // Check for empty message text
            if (String.IsNullOrEmpty(messageText))
            {
                EventLogProvider.LogEvent(EventType.ERROR, "TwilioSMS", "SEND", "Message Text is empty.  No message text is assigned to the Twilio Send SMS Automation Action in the workflow.");
                return;
            }

            // Get Values from Site Settings
            String accountSID = SettingsKeyInfoProvider.GetValue(SiteContext.CurrentSiteName + ".TwilioAccountSid");
            String authToken = SettingsKeyInfoProvider.GetValue(SiteContext.CurrentSiteName + ".TwilioAuthToken");
            String fromPhoneNumber = SettingsKeyInfoProvider.GetValue(SiteContext.CurrentSiteName + ".TwilioFromPhoneNumber");


            // Check for empty Account SID
            if (String.IsNullOrEmpty(accountSID))
            {
                EventLogProvider.LogEvent(EventType.ERROR, "TwilioSMS", "SEND", "Twilio Account SID is empty.  Review the Twilio configuration in Settings, under Integrations.");
                return;
            }

            // Check for empty Auth Token
            if (String.IsNullOrEmpty(authToken))
            {
                EventLogProvider.LogEvent(EventType.ERROR, "TwilioSMS", "SEND", "Twilio Auth Token is empty.  Review the Twilio configuration in Settings, under Integrations.");
                return;
            }

            // Check for empty From Phone Number
            if (String.IsNullOrEmpty(fromPhoneNumber))
            {
                EventLogProvider.LogEvent(EventType.ERROR, "TwilioSMS", "SEND", "Twilio Phone Number is empty.  Review the Twilio configuration in Settings, under Integrations.");
                return;
            }

            
            try
            {
                // Initilize Twilio Client with AccountSID and AuthToken from Settings
                TwilioClient.Init(accountSID, authToken);

                // Create Message to send to Twilio
                var message = MessageResource.Create(
                    to: new PhoneNumber(toNumber),
                    from: new PhoneNumber(fromPhoneNumber),
                    body: messageText);
            }
            catch (Exception excp)
            {
                // Log Exception generated with call to Twilio Service
                EventLogProvider.LogException("TwilioSMS", "SEND", excp);
            }
        }

    }
}
